local map = vim.keymap

-- Keymaps for better default experience
map.set({ 'n', 'v' }, '<Space>', '<Nop>', { silent = true })

-- end of line semicolon and comma
map.set('n', '<leader>;', 'A;<ESC>')
map.set('n', '<leader>,', 'A,<ESC>')


-- Remap for dealing with word wrap
-- map.set('n', 'k', "v:count == 0 ? 'gk' : 'k'", { expr = true, silent = true })
-- map.set('n', 'j', "v:count == 0 ? 'gj' : 'j'", { expr = true, silent = true })

-- Remaining cursor
map.set('n', 'J', 'mzJ`z')
map.set('n', '<C-d>', '<C-d>zz')
map.set('n', '<C-u>', '<C-u>zz')
map.set('n', 'n', 'nzzzv')
map.set('n', 'N', 'Nzzzv')

-- COPY PASTE
-- Moving lines
map.set("v", "J", ":m '>+1<CR>gv=gv")
map.set("v", "K", ":m '<-2<CR>gv=gv")
-- Paste and delete to void register
map.set('x', '<leader>p', '"_dP')
map.set({'n', 'v'}, '<leader>y', '"+y')
map.set('n', '<leader>Y', '"+Y')

-- Deleting to void
map.set({'n', 'v'}, '<leader>d', '"_d')

-- UTIL
-- replace current word
map.set("n", "<leader>s", [[:%s/\<<C-r><C-w>\>/<C-r><C-w>/gI<Left><Left><Left>]])
-- make executable
map.set("n", "<leader>x", "<cmd>!chmod +x %<CR>", { silent = true })

-- for vertical escape
map.set('i', '<C-c>', '<ESC>')

-- exit terminal mode
map.set('t', '<ESC>', '<C-\\><C-n>')

-- WINDOWS
map.set('n', '<C-p>', '<C-w>v') -- vertical split
map.set('n', '<C-i>', '<C-w>s') -- horizontal split
-- moving
map.set('n', '<C-h>', '<CMD>NvimTmuxNavigateLeft<CR>')
map.set('n', '<C-j>', '<CMD>NvimTmuxNavigateDown<CR>')
map.set('n', '<C-k>', '<CMD>NvimTmuxNavigateUp<CR>')
map.set('n', '<C-l>', '<CMD>NvimTmuxNavigateRight<CR>')
-- map.set('n', '<C-\\>', '<Cmd>NvimTmuxNavigateLastActive<CR>')
-- map.set('n', '<C-Space>', '<Cmd>NvimTmuxNavigateNext<CR>')

-- Diagnostic keymaps
map.set('n', '[d', vim.diagnostic.goto_prev)
map.set('n', ']d', vim.diagnostic.goto_next)
map.set('n', '<leader>e', vim.diagnostic.open_float)
map.set('n', '<leader>q', vim.diagnostic.setloclist)

-- BUFFERLINE
map.set('n', 'H', '<CMD>BufferLineCyclePrev<CR>', { desc = '[?] Go to previous BufferLine Buffer' }) -- Go to previous tab
map.set('n', 'L', '<CMD>BufferLineCycleNext<CR>', { desc = '[?] Go to next BufferLine Buffer' }) -- Go to next tab
map.set('n', '|', '<CMD>BufferLinePick<CR>', { desc = '[?] Go to specific Buffer, with selecting' }) -- pick tab
map.set('n', 'X', '<CMD>bp<bar>sp<bar>bn<bar>bd!<CR>') -- close tab

-- NVIM TREE
map.set('n', '<C-n>', '<CMD>NvimTreeToggle<CR>')
map.set('n', '<C-t>', '<CMD>NvimTreeOpen<CR>')

-- dap
map.set('n', '<F5>', '<CMD>lua require"dap".continue()<CR>')
map.set('n', '<F10>', '<CMD>lua require"dap".step_over()<CR>')
map.set('n', '<F11>', '<CMD>lua require"dap".step_into()<CR>')
map.set('n', '<F12>', '<CMD>lua require"dap".step_out()<CR>')
map.set('n', '<leader>b', '<CMD>lua require"dap".toggle_breakpoint()<CR>')
map.set('n', '<leader>B', '<CMD>lua require"dap".set_breakpoint(vim.fn.input("Breakpoint condition: "))<CR>')
map.set('n', '<leader>lp', '<CMD>lua require"dap".set_breakpoint(nil, nil, vim.fn.input("Log point message: "))<CR>')
map.set('n', '<leader>dr', '<CMD>lua require"dap".repl.open()<CR>')

-- dapui 
map.set('n', '<F6>', '<CMD>lua require"dapui".close()<CR>')
map.set('n', '<F7>', '<CMD>lua require"dapui".toggle()<CR>')
map.set('n', '<F8>', '<CMD>lua require"dapui".float_element(<element ID>, <optional settings>)<CR>')
map.set({ 'n', 'v' },'<F9>', '<CMD>lua requir("dapui".eval(<expression>)<CR>')

-- TELESCOPE
map.set('n', '<leader>?', require('telescope.builtin').oldfiles, { desc = '[?] Find recently opened files' })
map.set('n', '<leader><space>', require('telescope.builtin').buffers, { desc = '[ ] Find existing buffers' })
map.set('n', '<leader>/', function()
  -- You can pass additional configuration to telescope to change theme, layout, etc.
  require('telescope.builtin').current_buffer_fuzzy_find(require('telescope.themes').get_dropdown {
    winblend = 0,
    previewer = false,
  })
end, { desc = '[/] Fuzzily search in current buffer]' })

map.set('n', '<leader>sf', require('telescope.builtin').find_files, { desc = '[S]earch [F]iles' })
map.set('n', '<leader>sh', require('telescope.builtin').help_tags, { desc = '[S]earch [H]elp' })
map.set('n', '<leader>sw', require('telescope.builtin').grep_string, { desc = '[S]earch current [W]ord' })
map.set('n', '<leader>sg', require('telescope.builtin').live_grep, { desc = '[S]earch by [G]rep' })
map.set('n', '<leader>sd', require('telescope.builtin').diagnostics, { desc = '[S]earch [D]iagnostics' })
map.set('n', '<leader>ff', require('telescope.builtin').git_files, { desc = 'Find git files' })

-- UNDOTREE
map.set('n', '<leader><F5>', vim.cmd.UndotreeToggle)
