local opt = vim.opt
-- local g = vim.g
local o = vim.o
local wo = vim.wo

-- Colorcolumn
o.colorcolumn = "79"

-- Set highlight on search
o.hlsearch = false

-- Make line numbers default
wo.number = true -- activate numbers
opt.nu = true -- right line number
opt.relativenumber = true -- relative

-- keep cursor in the middle of the screen
opt.scrolloff = 25

-- TABS
o.expandtab = true
o.tabstop = 4
o.shiftwidth = 4
opt.expandtab = true
opt.autoindent = true

-- search settings
opt.ignorecase = true
opt.smartcase = true
opt.incsearch = true

-- appearance
o.termguicolors = true
opt.background = "dark"

-- Enable mouse mode
o.mouse = "a"

-- Enable break indent
o.breakindent = true

-- Save undo history
o.undofile = true

-- Case insensitive searching UNLESS /C or capital in search
o.ignorecase = true
o.smartcase = true

-- UNDODIR
opt.undodir = os.getenv("HOME") .. "/.vim/undodir"
opt.undofile = true

-- Decrease update time
o.updatetime = 250
wo.signcolumn = "yes"

-- Set completeopt to have a better completion experience
o.completeopt = "menuone,noselect"

-- [[ Highlight on yank ]]
-- See `:help vim.highlight.on_yank()`
local highlight_group = vim.api.nvim_create_augroup("YankHighlight", { clear = true })
vim.api.nvim_create_autocmd("TextYankPost", {
	callback = function()
		vim.highlight.on_yank()
	end,
	group = highlight_group,
	pattern = "*",
})
