local M = {}

function M.setup()
  local status_ok, alpha = pcall(require, "alpha")
  if not status_ok then
    return
  end

  local dashboard = require "alpha.themes.dashboard"
  dashboard.section.header.val = {
    [[]],
    [[███╗   ██╗ ███████╗ ██████╗  ██╗   ██╗ ██╗ ███╗   ███╗]],
    [[████╗  ██║ ██╔════╝██╔═══██╗ ██║   ██║ ██║ ████╗ ████║]],
    [[██╔██╗ ██║ █████╗  ██║   ██║ ██║   ██║ ██║ ██╔████╔██║]],
    [[██║╚██╗██║ ██╔══╝  ██║   ██║ ╚██╗ ██╔╝ ██║ ██║╚██╔╝██║]],
    [[██║ ╚████║ ███████╗╚██████╔╝  ╚████╔╝  ██║ ██║ ╚═╝ ██║]],
    [[╚═╝  ╚═══╝ ╚══════╝ ╚═════╝    ╚═══╝   ╚═╝ ╚═╝     ╚═╝]],
    [[]],
  }

  dashboard.section.buttons.val = {
    dashboard.button("f", codevim.icons.ui.FindFile .. "  Find File", "<CMD>Telescope find_files<CR>"),
    dashboard.button("n", codevim.icons.ui.NewFile .. "  New File", "<CMD>ene!<CR>"),
    dashboard.button("p", codevim.icons.ui.Project .. "  Projects ", "<CMD>Telescope projects<CR>"),
    dashboard.button("b", codevim.icons.ui.FolderOpen .. "  File Browser", '<CMD>Telescope file_browser<CR>'),
    dashboard.button("r", codevim.icons.ui.History .. "  Recent files", "<CMD>Telescope oldfiles <CR>"),
    dashboard.button("t", codevim.icons.ui.FindText .. "  Find Text", "<CMD>Telescope live_grep<CR>"),
    dashboard.button("c", codevim.icons.ui.Gear .. "  Configuration", "<CMD>cd ~/.config/nvim/<CR><CMD>e init.lua<CR>"),
    dashboard.button("q", codevim.icons.ui.Close .. "  Quit", "<CMD>qall<CR>"),
  }

  local function footer()
    -- Number of plugins
    local datetime = os.date "%d-%m-%Y %H:%M:%S"
    local plugins_text = "   "
        .. "   v"
        .. vim.version().major
        .. "."
        .. vim.version().minor
        .. "."
        .. vim.version().patch
        .. "   "
        .. datetime
    -- Quote
    local fortune = require "alpha.fortune"
    local quote = table.concat(fortune(), "\n")

    return plugins_text .. "\n" .. quote
  end

  dashboard.section.buttons.opts = {
    hl_shortcut = "Include",
    spacing = 1,
  }

  dashboard.section.footer.opts = {
    position = "center",
    hl = "Number",
  }
  dashboard.section.header.opts = {
    position = "center",
    hl = "Label",
  }
  dashboard.section.footer.val = footer()

  dashboard.section.footer.opts.hl = "Type"
  dashboard.section.header.opts.hl = "Include"
  dashboard.section.buttons.opts.hl = "Keyword"

  dashboard.opts.opts.noautocmd = true
  alpha.setup(dashboard.opts)
end

return M
